<?php

namespace OCA\PahekoFSpace\AppInfo;

use OCA\PahekoFSpace\Listener\UserChangeListener;
use OCP\Accounts\UserUpdatedEvent;
use OCP\AppFramework\App;
use OCP\AppFramework\Bootstrap\IBootContext;
use OCP\AppFramework\Bootstrap\IBootstrap;
use OCP\AppFramework\Bootstrap\IRegistrationContext;
use OCP\IGroupManager;
use OCP\INavigationManager;
use OCP\IURLGenerator;
use OCP\IUserSession;
use OCP\User\Events\BeforeUserDeletedEvent;
use OCP\User\Events\UserChangedEvent;
use OCP\User\Events\UserCreatedEvent;

class Application extends App implements IBootstrap {
	public const APP_ID = 'paheko_fspace';

	public function __construct(array $urlParams = []) {
		parent::__construct(self::APP_ID, $urlParams);
	}


	public function register(IRegistrationContext $context): void {
		require_once __DIR__ . '/../../vendor/autoload.php';
		$context->registerEventListener(UserCreatedEvent::class, UserChangeListener::class);
		$context->registerEventListener(UserChangedEvent::class, UserChangeListener::class);
		$context->registerEventListener(BeforeUserDeletedEvent::class, UserChangeListener::class);
		$context->registerEventListener(UserUpdatedEvent::class, UserChangeListener::class);
	}

	public function boot(IBootContext $context): void {
		$context->injectFn([$this, 'registerNavigationEntry']);
	}

	public function registerNavigationEntry(
		IUserSession $userSession,
		IGroupManager $groupManager,
		INavigationManager $navigationManager,
		IURLGenerator $url): void {

		$user = $userSession->getUser();
		if (!$user) {
			return;
		}
		if ($groupManager->isAdmin($user->getUID()) || $this->canAccessPaheko($groupManager, $user->getUID())) {
			$navigationManager->add(function () use ($url) {
				$image = $url->imagePath('paheko_fspace', 'paheko.svg');
				$href = $url->linkToRoute('paheko_fspace.page.index');

				return [
					'id' => 'paheko_fspace.admin',
					'href' => $href,
					'icon' => $image,
					'name' => 'Paheko',
					'order' => 99
				];
			});
		}
	}

	private function canAccessPaheko(IGroupManager $groupManager, string $userId): bool {
		return ($groupManager->groupExists('paheko_compta') && $groupManager->isInGroup($userId, 'paheko_compta')) || ($groupManager->groupExists('paheko_membres') && $groupManager->isInGroup($userId, 'paheko_fspace'));
	}
}
