<?php

namespace OCA\PahekoFSpace\Command;

use OCA\PahekoFSpace\AppInfo\Application;
use OCP\IAppConfig;
use ParagonIE\Paseto\Keys\SymmetricKey;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;

class GenerateSharedKey extends Command {
	public function __construct(
		private readonly IAppConfig $config,
	) {
		parent::__construct();
	}

	protected function configure(): void {
		$this->setName('paheko_fspace:generate-shared-key')
			->setDescription('Generate shared PASETO key between Paheko and Nextcloud')
			->addOption('no-save', null, InputOption::VALUE_NONE, "Don't save the generated key")
			->addOption('force', 'f', InputOption::VALUE_NONE, 'Force overwrite the existing saved shared key')
		;
	}

	protected function execute(InputInterface $input, OutputInterface $output): int {
		$sharedKey = SymmetricKey::generate()->encode();
		$noSave = $input->getOption('no-save');
		$force = $input->getOption('force');
		if (!$noSave) {
			$existingKey = $this->config->getValueString(Application::APP_ID, 'api_shared_key');
			if (!$existingKey || $force) {
				$this->config->setValueString(Application::APP_ID, 'api_shared_key', $sharedKey);
				if ($force) {
					$output->writeln('Shared key generated and replaced: ' . $sharedKey);
				} else {
					$output->writeln('Shared key generated and saved: ' . $sharedKey);
				}
				return Command::SUCCESS;
			}
		}
		$output->writeln('Shared key generated: ' . $sharedKey);
		return Command::SUCCESS;
	}
}
