<?php

namespace OCA\PahekoFSpace\Listener;

use OCA\PahekoFSpace\Service\SyncUsers;
use OCP\Accounts\UserUpdatedEvent;
use OCP\EventDispatcher\Event;
use OCP\EventDispatcher\IEventListener;
use OCP\User\Events\BeforeUserDeletedEvent;
use OCP\User\Events\UserChangedEvent;
use OCP\User\Events\UserCreatedEvent;

/** @template-implements IEventListener<UserCreatedEvent|UserUpdatedEvent|UserChangedEvent|BeforeUserDeletedEvent> */
class UserChangeListener implements IEventListener {
	public function __construct(
		private readonly SyncUsers $syncUsers,
	) {
	}

	public function handle(Event $event): void {
		try {
			if ($event instanceof UserCreatedEvent) {
				$user = $event->getUser();
				$this->syncUsers->createUser($user);
			} elseif ($event instanceof UserUpdatedEvent || $event instanceof UserChangedEvent) {
				// change of email or display name
				$user = $event->getUser();
				$this->syncUsers->updateUser($user);
			} elseif ($event instanceof BeforeUserDeletedEvent) {
				$user = $event->getUser();
				$this->syncUsers->deleteUser($user);
			}
		} catch (\Exception $e) {
			// do nothing, things are already logged
			// and we don't want to block the action
		}
	}
}
