<?php

/**
 * SPDX-FileCopyrightText: 2024 Framasoft <https://framasoft.org>
 * SPDX-FileContributor: Thomas Citharel <thomas.citharel@framasoft.org>
 *
 * SPDX-License-Identifier: AGPL-3.0-only
 */

declare(strict_types=1);

namespace OCA\PahekoFSpace\Tests\Service;

use OCA\PahekoFSpace\AppInfo\Application;
use OCA\PahekoFSpace\Command\GenerateSharedKey;
use OCP\IAppConfig;
use ParagonIE\Paseto\Keys\Base\SymmetricKey as BaseSymmetricKey;
use ParagonIE\Paseto\Keys\SymmetricKey;
use PHPUnit\Framework\Attributes\DataProvider;
use PHPUnit\Framework\MockObject\MockObject;
use Symfony\Component\Console\Tester\CommandTester;
use Test\TestCase;

class GenerateSharedKeyTest extends TestCase {

	private IAppConfig|MockObject $config;
	private CommandTester $commandTester;

	public function setUp(): void {
		parent::setUp();

		$this->config = $this->createMock(IAppConfig::class);

		$command = new GenerateSharedKey($this->config);
		$this->commandTester = new CommandTester($command);
	}

	#[DataProvider('dataTestGenerateSharedKeyDataProvider')]
	public function testGenerateSharedKey(bool $noSave, bool $force, bool $hasExistingKey, bool $setKey, string $pattern) {

		$this->config->expects($noSave ? $this->never() : $this->once())->method('getValueString')->with(Application::APP_ID, 'api_shared_key')->willReturn($hasExistingKey ? 'a key' : '');
		$this->config->expects($setKey ? $this->once() : $this->never())->method('setValueString')->with(Application::APP_ID, 'api_shared_key', $this->callback(function (string $value) {
			return (SymmetricKey::fromEncodedString($value) instanceof BaseSymmetricKey);
		}));

		$this->commandTester->execute([
			'--no-save' => $noSave,
			'--force' => $force,
		]);
		$output = $this->commandTester->getDisplay();

		$this->commandTester->assertCommandIsSuccessful();
		$this->assertStringContainsString($pattern, $output);
	}

	public static function dataTestGenerateSharedKeyDataProvider(): array {
		return [
			// no-save, force, get, set, pattern
			[true, false, true, false, 'Shared key generated: '],
			[false, false, true, false, 'Shared key generated: '],
			[false, true, true, true, 'Shared key generated and replaced:'],
			[true, false, false, false, 'Shared key generated: '],
			[false, false, false, true, 'Shared key generated and saved: '],
			[false, true, false, true, 'Shared key generated and replaced:']
		];
	}
}
