<?php

/**
 * SPDX-FileCopyrightText: 2024 Framasoft <https://framasoft.org>
 * SPDX-FileContributor: Thomas Citharel <thomas.citharel@framasoft.org>
 *
 * SPDX-License-Identifier: AGPL-3.0-only
 */

declare(strict_types=1);

namespace OCA\PahekoFSpace\Tests\Listeners;

use OCA\PahekoFSpace\Listener\UserChangeListener;
use OCA\PahekoFSpace\Service\SyncUsers;
use OCP\Accounts\UserUpdatedEvent;
use OCP\EventDispatcher\Event;
use OCP\IUser;
use OCP\User\Events\BeforeUserDeletedEvent;
use OCP\User\Events\UserChangedEvent;
use OCP\User\Events\UserCreatedEvent;
use PHPUnit\Framework\MockObject\MockObject;
use Test\TestCase;

class UserChangeListenerTest extends TestCase {

	private SyncUsers|MockObject $syncUsers;
	private UserChangeListener $listener;

	public function setUp(): void {
		parent::setUp();
		$this->syncUsers = $this->createMock(SyncUsers::class);

		$this->listener = new UserChangeListener($this->syncUsers);
	}

	public function testHandleUnrelated(): void {
		$event = new Event();
		$this->syncUsers->expects($this->never())->method($this->anything());
		$this->listener->handle($event);
	}

	public function testHandleUserCreation(): void {
		$user = $this->createMock(IUser::class);
		$event = new UserCreatedEvent($user, '');
		$this->syncUsers->expects($this->once())->method('createUser')->with($user);
		$this->syncUsers->expects($this->never())->method('updateUser');
		$this->syncUsers->expects($this->never())->method('deleteUser');
		$this->listener->handle($event);
	}

	public function testHandleUserDisplayNameUpdate(): void {
		$user = $this->createMock(IUser::class);
		$event = new UserChangedEvent($user, 'displayName', 'new name', 'old name');
		$this->syncUsers->expects($this->never())->method('createUser');
		$this->syncUsers->expects($this->once())->method('updateUser')->with($user);
		$this->syncUsers->expects($this->never())->method('deleteUser');
		$this->listener->handle($event);
	}

	public function testHandleUserEmailUpdate(): void {
		$user = $this->createMock(IUser::class);
		$event = new UserUpdatedEvent($user, []);
		$this->syncUsers->expects($this->never())->method('createUser');
		$this->syncUsers->expects($this->once())->method('updateUser')->with($user);
		$this->syncUsers->expects($this->never())->method('deleteUser');
		$this->listener->handle($event);
	}

	public function testHandleUserDeletion(): void {
		$user = $this->createMock(IUser::class);
		$event = new BeforeUserDeletedEvent($user);
		$this->syncUsers->expects($this->never())->method('createUser');
		$this->syncUsers->expects($this->never())->method('updateUser');
		$this->syncUsers->expects($this->once())->method('deleteUser')->with($user);
		$this->listener->handle($event);
	}
}
